libname bgt 'd:\publications\bugt\SAS';
libname wrds 'd:\wrds';
options ps=3000 ls=150;

%macro winsor(dsetin=, dsetout=, byvar=, vars=, type=, pctl=1 99);

%if &dsetout = %then %let dsetout = &dsetin;
    
%let varL=;
%let varH=;
%let xn=1;

%do %until ( %scan(&vars,&xn)= );
    %let token = %scan(&vars,&xn);
    %let varL = &varL &token.L;
    %let varH = &varH &token.H;
    %let xn=%EVAL(&xn + 1);
%end;

%let xn=%eval(&xn-1);

data xtemp;
    set &dsetin;
    run;

%let dropvar = ;
%if &byvar = none %then %do;

    data xtemp;
        set xtemp;
        xbyvar = 1;
        run;

    %let byvar = xbyvar;
    %let dropvar = xbyvar;

%end;

proc sort data = xtemp;
    by &byvar;
    run;

proc univariate data = xtemp noprint;
    by &byvar;
    var &vars;
    output out = xtemp_pctl PCTLPTS = &pctl PCTLPRE = &vars PCTLNAME = L H;
    run;

data &dsetout;
    merge xtemp xtemp_pctl;
    by &byvar;
    array trimvars{&xn} &vars;
    array trimvarl{&xn} &varL;
    array trimvarh{&xn} &varH;

    do xi = 1 to dim(trimvars);

        %if &type = winsor %then %do;
            if trimvars{xi} ne . then do;
              if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = trimvarl{xi};
              if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = trimvarh{xi};
            end;
        %end;

        %else %do;
            if trimvars{xi} ne . then do;
              if (trimvars{xi} < trimvarl{xi}) then delete;
              if (trimvars{xi} > trimvarh{xi}) then delete;
            end;
        %end;

    end;
    drop &varL &varH &dropvar xi;
    run;
%mend winsor;




**************************************************;
**												**;
** STEP #1:		IMPORT THOMSON CONF CALL DATA	**;
**												**;
**************************************************;

*CLEAN AND FORCE UNIQUE DATA PER GVKEY-CALLDATE;
proc freq data=bgt.measures_confcall; tables call_date; format call_date mmddyyn8.; run;
data measuresCALL; set bgt.measures_confcall; 
if 2011>=year(call_date);
fog_pres=fog_comp_pres;	fog_anal=fog_anal_qa; fog_resp=fog_comp_qa;  	
keep permno datadate gvkey rdq call_date fog_pres fog_anal fog_resp;
run; *86321;
proc sort data=measuresCALL nodupkey; by gvkey rdq call_date descending datadate; run;
proc sort data=measuresCALL nodupkey; by gvkey rdq call_date ; run;
proc sort data=measuresCALL nodupkey; by gvkey rdq ; run; *85525;
proc sort data=measuresCALL nodupkey; by gvkey call_date ; run; *85518;



**********************************************************;
**														**;
** STEP #2:	CLEAN & ADD IBES ANALYST DATA 		 		**;
**														**;
**********************************************************;

*quarterly EPS forecasts;
proc sort data=bgt.IBESstatsum75to12_US_unadj out=forecasts (drop= USFIRM OFTIC CURCODE) nodupkey; 
where 	trim(fpi) ="6" and 
		upcase(trim(fiscalp))="QTR" and 
		upcase(trim(measure))="EPS" and 
		numest>=1 and USFIRM=1 and
		upcase(trim(curcode))="USD" and
		not missing(medest) and
		2012>=year(statpers)>=2000;  
by ticker fpedats statpers estflag ; run; 
proc sort data=forecasts (drop= estflag) nodupkey; by ticker fpedats statpers; run; *612986 keep primary, if none keeps 2ndry;

*clean actual EPS data;
proc sort data=bgt.IBESactuals75to12_US_unadj out=actuals (drop= USFIRM OFTIC pdicity measure) nodupkey; 
where 	upcase(trim(pdicity))="QTR" and 
		upcase(trim(measure))="EPS" and 
		USFIRM=1 and
		not missing(value) and
		2012>=year(pends)>=2000;  
by ticker pends anndats; run; 
proc sort data=actuals nodupkey; by ticker pends; run; *287805;

*clean IBES shares outstanding;
proc sort data=bgt.IBESpricing75to12_US_unadj out=price (drop= USFIRM OFTIC measure) nodupkey; 
where	USFIRM=1 and 	
		upcase(trim(measure))="EPS" and 
		upcase(trim(curcode))="USD" and
		not missing(price) and not missing(shout) and
		2012>=year(statpers)>=2000; 
by ticker statpers descending prdays; run;
proc sort data=price; by ticker statpers; run; *682896;

*create IBES dataset with forecast-actuals-sharesout;
proc sql; 
create table addactuals as select a.*, b.value as IBES_EPS, b.anndats 
from forecasts a inner join actuals b on a.ticker=b.ticker and a.fpedats=b.pends; 

create table addprice as select a.*, b.shout as IBES_shout, b.price as IBES_price, b.prdays as IBESpricingdate
from addactuals a inner join price b on a.ticker=b.ticker and a.statpers=b.statpers; 
quit; *586353;

*takes latest forecast per fiscal-period and requires earnings announcement within 3 months of fiscal period end;
data addprice; set addprice; if 0<=intck('months',fpedats,anndats)<=3 and fpedats<anndats; run; *577749;
proc sort data=addprice nodupkey; by ticker anndats fpedats descending statpers; run; *577749;
proc sort data=addprice nodupkey; by ticker anndats fpedats; run; *197067;


************FETCH WRDS IBES-CRSP linking file*****************;
*WE SEEK A 1-to-1 MATCH BETWEEN IBES AND PERMNO;

proc sort data=bgt.iclink2013 out=t1 nodupkey; where score<=1; by permno score ticker; run; 
*16955...<=1 is CUSIPs and CUSIP-DATES match, best possible links;

proc sql; create table t2 as select a.* from t1 a group by permno having min(score)=score; quit; 
*16907...keep only those permnos with lowest score;

data t2; set t2; by permno score; if first.permno and last.permno then output; run; 
*16706...if among the lowest scores & 1 PERMNO is matched to more than 1 TICK then remove;

proc sql; create table t3 as select a.* from t2 a group by ticker having min(score)=score; quit; 
*16597...keep only those tickers with lowest score;

data t3; set t3; by ticker score; if first.ticker and last.ticker then output; run; 
*15910...if among the lowest scores & 1 TICK is matched to more than 1 PERMNO then remove;

proc sort data=t3 nodupkey; by permno; run; *kill 0;
proc sort data=t3 out=ilink nodupkey; by ticker; run; *kill 0...since unique permnos and unique tickers we have a unique 1-1 match; 

*ADD CRSP PERMNO TO IBES;
proc sql; 
create table addpermno as 
select 	a.ticker, fpedats, anndats, IBES_EPS, medest as IBES_EXP_EPS, stdev as IBES_std, numest, IBES_shout, IBES_price, IBESpricingdate, b.permno 
from addprice a inner join ilink b on a.ticker=b.ticker;
quit; *186874;
proc sort data=addpermno out=IBESunadj nodupkey; by permno fpedats; where 2011>=year(fpedats)>=2000;  run; *173326;

*ADD IBES ANALYST DATA TO MAIN DATASET;
proc sql; 
create table CONFCALL as 
select 	a.*, b.IBES_EPS, b.IBES_EXP_EPS, b.IBES_shout, b.IBES_std, b.numest as Nanalysts, b.IBES_price
from measurescall a left join IBESunadj b on a.permno=b.permno and intck('month',a.rdq,b.anndats)=0 and intck('month',a.datadate,b.fpedats)=0;
quit; *85518;
proc sort data=CONFCALL nodupkey; by gvkey call_Date; run; *85518;






**********************************************************;
**														**;
** STEP #3:	CLEAN & ADD COMPUSTAT DATA	 		 		**;
**														**;
**********************************************************;

*CLEAN QUARTERLY COMPUSTAT;
data mycompustat;  set wrds.CompustatQ61to11; 
where fyr > 0 and indfmt='INDL' and datafmt='STD' and popsrc='D' and consol='C' and fyearq>=1990;  
lagfyearq=fyearq;	lagfqtr=fqtr-1;
aqcy=sum(aqcy,0); 	capxy=sum(capxy,0);	sstky=sum(sstky,0);	dltisy=sum(dltisy,0);
run; 
proc sort data=mycompustat nodupkey; by gvkey fyearq fqtr; run;
data addlag; set mycompustat; by gvkey fyearq fqtr;
if gvkey=lag1(gvkey) and fyearq=lag1(fyearq) and fqtr = (1+lag1(fqtr)) then m1q=1; else m1q=.;
acq=dif(aqcy)*m1q; 			if lagfqtr=0 then acq=aqcy;
capex=dif(capxy)*m1q; 		if lagfqtr=0 then capex=capxy;
stockiss=dif(sstky)*m1q; 	if lagfqtr=0 then stockiss=sstky ;
debtiss=dif(dltisy)*m1q; 	if lagfqtr=0 then debtiss=dltisy;
run;

*ADD COMPUSTAT QUARTER Q;
proc sql; 
create table addCompu as select unique a.*, b.fyearq, b.fqtr, b.ibq, b.atq as ta, b.ceqq as bv, b.prccq*b.cshoq as mv_compu, 
spiq as specitems, xrdq as rd, ppentq as NPPE, sum(dlttq,dlcq,0)/atq as LEV, prccq, cshoq, acq, capex, stockiss, debtiss,
sign(sum(abs(sign(GDWLIPQ)),abs(sign(GDWLIAQ)),abs(sign(GDWLIEPSQ)),abs(sign(GDWLIDQ)),0)) as GoodwillBreakout,
sign(sum(abs(sign(RCPQ)),abs(sign(RCAQ)),abs(sign(RCEPSQ)),abs(sign(RCDQ)),0)) as RestructingBreakout
from CONFCALL a inner join addlag b on a.gvkey=b.gvkey and a.rdq=b.rdq and a.datadate=b.datadate
and b.indfmt="INDL" and b.consol="C" and b.popsrc="D" and b.datafmt="STD" and nmiss(b.ibq, b.atq, b.ceqq)=0;
quit; *85229;
data addcompu; set addcompu;
lagfyearq=fyearq;	lagfqtr=fqtr-1;
if lagfqtr=0 then do; lagfqtr=4; lagfyearq=fyearq-1; end;

futfyearq=fyearq;	futfqtr=fqtr+1;
if lagfqtr=5 then do; futfqtr=1; futfyearq=fyearq+1; end;
run;
proc sort data=addcompu nodupkey; by gvkey call_Date descending fyearq descending fqtr; run; 
proc sort data=addcompu nodupkey; by gvkey call_Date; 									run; *85215;

*ADD COMPUSTAT QUARTER Q-1 & Q-4;
proc sql;
create table addCompu as select unique a.*, b.ibq as ibq_m1, b.atq as ta_m1,  b.prccq*b.cshoq as mv_compu_m1, b.ceqq as bv_m1, spiq as specitems_m1  
from addCompu a left  join mycompustat b on a.gvkey=b.gvkey and a.lagfyearq=b.fyearq and a.lagfqtr=b.fqtr and nmiss(b.atq,b.ibq,b.ceqq)=0;

create table addCompu as select unique a.*, b.ibq as ibq_m4, b.atq as ta_m4,  b.prccq*b.cshoq as mv_compu_m4, b.ceqq as bv_m4, spiq as specitems_m4  
from addCompu a left  join mycompustat b on a.gvkey=b.gvkey and a.fyearq=b.fyearq+1 and a.fqtr=b.fqtr and nmiss(b.atq,b.ibq,b.ceqq)=0;
quit; *85215;

*ADD COMPUSTAT Y-1;
proc sql;
create table controlsANN as select unique a.*, b.ppent as ppenta
from addCompu a left join bgt.CompustatA61to11 b on a.gvkey=b.gvkey and a.fyearq=b.fyear+1 and nmiss(b.at,b.ib,b.ceq)=0
and b.indfmt="INDL" and b.consol="C" and b.popsrc="D" and b.datafmt="STD";

create table controlsANN as select unique a.*, std(b.sale/b.at) as sigmasales, std((b.oancf - b.xidoc)/b.at) as sigmacfo, std(b.IB/b.at) as sigmaEarn  
from controlsANN a left join bgt.compustatA61to11 b on a.gvkey=b.gvkey and a.fyearq-1>=b.fyear>=a.fyearq-5 
group by a.gvkey, a.call_Date having count(b.fyear)>=5; 
quit; 
proc sort data=controlsANN nodupkey; by gvkey call_Date; run; *84389;






**********************************************************;
**														**;
** STEP #4: ADD CRSP							 		**;
**														**;
**********************************************************;

*ADD CRSP-SIZE;
proc sql;
create table addCRSP as select unique a.*, shrcd as shrcd_m4, siccd as siccd_m4,
abs(prc*shrout)/1000 as CRSPMV_m4, abs(prc)/cfacpr as PRC_m4, shrout*cfacshr as SHROUT_m4, exchcd as exchcd_m4, prc as PRC_unadj_m4
from controlsANN a left join wrds.crspmonthly70to12dlst b on (a.permno = b.permno) 	
and intck('month',datadate,date)=-3 and abs(prc)>0 and shrout>0 and cfacpr>0;

create table addCRSP as select unique a.*, shrcd, siccd,
abs(prc*shrout)/1000 as CRSPMV, abs(prc)/cfacpr as PRC, shrout*cfacshr as SHROUT, exchcd, prc as PRC_unadj
from addCRSP a left join wrds.crspmonthly70to12dlst b on (a.permno = b.permno) 	
and intck('month',datadate,date)=0 and abs(prc)>0 and shrout>0 and cfacpr>0;
quit;
proc sort data=addcrsp nodupkey; by gvkey call_date ; run; *84389;

*COMPUTE IDVOL;
proc sql;
create table rets as select unique gvkey, call_date, b.date, b.ret
from controlsANN a inner join wrds.crspdsfdaily70to12 b 
on (a.permno = b.permno) and -2<=intck('month',a.datadate,b.date)<=0 and nmiss(b.vol,b.ret,b.prc)=0 and ret>-1;

create table rets as select a.*, a.ret-b.rf as rirf, b.mktrf, b.smb, b.hml, b.umd
from rets a inner join bgt.FFandUMDdaily70to12 b on a.date=b.date; 
quit; 
options nonotes;
proc reg data=rets noprint ; by gvkey call_date; model rirf = mktrf smb hml; output out=resids r=resid; quit;
option notes;
proc sql; 
create table id as select unique gvkey, call_date, std(resid) as idvol3m, (exp(sum(log(1+ret)))-1) as BHR3m 
from resids a where not missing(resid) group by gvkey, call_Date;
quit; 

*COMPUTE ILLIQ;
proc sql;
create table dailyilliq as select unique a.gvkey, a.call_date, abs(b.ret)/(b.vol*abs(b.prc)) as Illiq, date, b.vol
from controlsANN a inner join wrds.crspdsfdaily70to12 b on (a.permno = b.permno) and nmiss(b.vol,b.ret,b.prc)=0 and 0<=intck('weekday',call_date,date)<=25;	
quit; 
proc univariate data=dailyilliq noprint; by gvkey call_date; var illiq; output out=illiq mean= illiq 	median= milliq; run; 

*MERGE;
proc sort data=addcrsp nodupkey; 		by gvkey call_Date; 	run; 
proc sort data=illiq nodupkey;  		by gvkey call_date; 	run; 
proc sort data=id nodupkey; 			by gvkey call_Date; 	run; 
data addcrsp; merge addcrsp illiq id; by gvkey call_date; illiq=illiq*1000000; run; *84389;





**********************************************************;
**														**;
** STEP #5: ADD SUPPLEMENTAL DATA	& CLEAN		 		**;
**														**;
**********************************************************;

*ADD SUPPLEMENTAL CALL DATA (e.g., JARGON, ETC);
proc sql; 
create table supp as select unique a.*, b.*
from addcrsp a inner join bgt.fog_data3 b on a.permno=b.permno and a.rdq=b.rdq and a.call_date=b.call_date and a.gvkey=b.gvkey
and nmiss(b.fog_comp_pres, b.fog_comp_qa, b.fog_anal_qa)=0;
quit;
proc sort data=supp  nodupkey; by gvkey call_Date; run; *81914;
data bgt.finalsample_JAR; set supp ; run;

data addcalldata; set bgt.finalsample_JAR; drop fog_pres fog_anal fog_resp;
if nmiss(illiq,IBES_EPS,IBES_EXP_EPS,nanalysts,ibes_std)=0; 

if (num_sentences_comp_qa+num_sentences_comp_pres)>0 then 
fog_comp=0.4 * (100*((num_complex_words_comp_qa+num_complex_words_comp_pres)/(num_words_comp_qa+num_words_comp_pres)) +
    ((num_words_comp_qa+num_words_comp_pres)/(num_sentences_comp_qa+num_sentences_comp_pres))); 
if not missing(fog_comp); 

*forward-looking;
if prop_fl_sents_comp_qa=. then prop_fl_sents_comp_qa=0;
if prop_fl_sents_comp_pres=. then prop_fl_sents_comp_pres=0;

*tone;
if negative_comp_qa=. then negative_comp_qa=0;
if positive_comp_qa=. then positive_comp_qa=0;
if negative_comp_pres=. then negative_comp_pres=0;
if positive_comp_pres=. then positive_comp_pres=0;

*jargon;
if num_jargon_words_comp_pres=. then num_jargon_words_comp_pres=0;
if num_jargon_words_comp_qa=. then num_jargon_words_comp_qa=0;

if IBES_EPS<0 then loss=1; else loss=0;
if 0<=IBES_EPS-IBES_EXP_EPS<=0.01 then smallbeat=1; else smallbeat=0;

*declare timeing of compustat variables;
ta=ta_m1; mv=mv_compu_m1; bv=bv_m1; idvol=idvol3m; momentum=bhr3m*100; coverage=nanalysts;

if mv>0 then do; IBESmv=((IBES_EPS-IBES_EXP_EPS)*IBES_SHOUT)/mv; itemsmv=sum(specitems,0)/mv; end;
if mv>0 then size=log(mv);
if mv>0 then BM=bv/mv;
invprc=1/prc_m4;
disp=(ibes_std*IBES_SHOUT)/mv;

acquisitions=acq/ta_m1;
capital=capex/ta_m1;

if nppe=. then nppe=ppenta;
capintensity=nppe/ta_m1;
research=sum(rd,0)/ta_m1;
financing=sum(stockiss,debtiss)/ta_m1;

if GoodwillBreakout=. then GoodwillBreakout=0;
if RestructingBreakout=. then RestructingBreakout=0;

if 2011>=year(call_date)>=2002;
if nmiss(size,bm,momentum,idvol,coverage,disp,ibesmv,loss,itemsmv,invprc,lev,
research,capintensity,sigmacfo,acquisitions,financing,capital,fog_comp_pres,fog_comp_qa,fog_anal_qa)=0;
run; *60172;

*produce unique PERMNOs;
proc sort data=addcalldata out=uniquefirmIDS (keep=permno) nodupkey; by permno; run;
proc print data=uniquefirmIDS noobs; run;

*STEP #6: BEGIN ANALYSIS;
